#ifndef VPAPPLICATION_H
#define VPAPPLICATION_H

#include <QObject>
#include <QAbstractAnimation>
#include <QStringList>
#include <QVariantMap>

class VPlayScheduler;
class QQmlEngine;
class System;
class GlobalKeyForwarder;
class VPlayFileUtils;
class VPlayUrlInterceptor;
class QFileSelector;

/**
 * Small utility class to synchronize the stepping with the framerate.
 */
class VPStepDriver : public QAbstractAnimation
{
  Q_OBJECT

public:
  explicit VPStepDriver(VPlayScheduler *scheduler);

  int duration() const;

protected:
  void updateCurrentTime(int currentTime);

private:
  VPlayScheduler *m_vplayScheduler;
};

class VPApplication : public QObject
{
  Q_OBJECT
public:
  explicit VPApplication(QObject *parent = 0);

  ~VPApplication();

  /*!
   * Makes V-Play available as "import VPlay 2.0".
   * Calls initializeEngine() and registerTypes() internally.
   */
  void initialize(QQmlEngine *engine);
  /*
    This may be called in future by a dynamic QQmlExtensionPlugin
   */
  void initializeEngine(QQmlEngine *engine, const char *uri);
  void registerTypes(const char *uri);


  /*!
   * Call to set a custom contentScale for testing different resources at startup.
   *
   * The fileSelectors are set based on this scale and the config.json information.
   * \param contentScale
   */
  void setContentScaleAndFileSelectors(qreal contentScale);

  /*!
   * Returns the contentScale that got calculated based on the screen size.
   * Useful for custom defining fileSelectors based on this contentScale.
   * For example if the max screen size is 960px, at a given scene width of 480 the contentScale is 2.
   */
  qreal contentScale() const;

  /*!
   * Returns the used scaleFactor for MultiResolutionImage components.
   * It gets calculated in setContentScaleAndFileSelectors().
   * For example if the max screen size is 2000px, at a given scene width of 480 the scaleFactorForImages is 4, because that was set as "internalContentScaleFactor" in contentScaleThresholds().
   */
  qreal scaleFactorForImages() const;

  /*!
   * Set a custom contentScaleThreshold map.
   * After setting a custom contentScaleThresholds map, you probably want to call \c setContentScaleAndFileSelectors(contentScale()); so the fileSelectors are updated.
   *
   * Example:
     \code

      QVariantMap customThresholds;
      QVariantMap sdMap;
      sdMap["scaleThreshold"] = 1;
      sdMap["internalContentScaleFactor"] = 1;
      customThresholds["sd"] = sdMap;
      QVariantMap hdMap;
      hdMap["scaleThreshold"] = 2;
      hdMap["internalContentScaleFactor"] = 2;
      customThresholds["hd"] = hdMap;
      QVariantMap hd2Map;
      hd2Map["scaleThreshold"] = 2.8;
      hd2Map["internalContentScaleFactor"] = 4;
      customThresholds["hd2"] = hd2Map;

      vplay.setContentScaleThresholds(customThresholds);
      vplay.setContentScaleAndFileSelectors(vplay.contentScale());

     \endcode
   */
  void setContentScaleThresholds(QVariantMap contentScaleThresholds);

  /*!
   * Returns the contentScaleThresholds. By default, it is set to
   *
    \code
     {
       "sd": { "scaleThreshold": 1, "internalContentScaleFactor": 1 },
       "hd": { "scaleThreshold": 2, "internalContentScaleFactor": 2 },
       "hd2": { "scaleThreshold": 2.8, "internalContentScaleFactor": 4}
     }
    \endcode
   *
   */
  const QVariantMap &contentScaleThresholds() const;

  /*!
   * Returns the current fileSelectorList. Useful for adding new fileSelectors to the existing ones.
   */
  const QStringList &fileSelectorList() const;
  /*!
   * Use this function to manually set a fileSelectorList.
   * \param selectorList
   */
  void setFileSelectorList(const QStringList &fileSelectorList);

  /*!
      This is only useful for running application with multiple GameWindow instances in it.
      It avoids creating a new window for each of the loaded GameWindow instance. This is especially important on Android, where only a single Window is allowed.
      This function is also useful, if you want to avoid on Desktop to create a new Window for each GameWindow instance in a multi-GameWindow launcher (like Box2dMultiExamplesLoader.qml).

      The reason for GameWindow instances being tricky are the different possibility to load the main qml file:
      \list
      \li QQmlApplicationEngine requires a QQuickWindow as root element with visible set to true to show something.
      \li QQuickWindow (such as in QtQuick2ControlsApplicationViewer.pri) also requires a Window or ApplicationWindow as root element (and sets visible to true by default).
      \li QQuickView (such as in QtQuick2ApplicationViewer.pri) does not allow a Window element as root element - in such cases you could also call this function.
      \endlist

   */
  void setGameWindowIsItem();

  const QString &mainQmlFileName() const;
  /*!
    Adds the \c assets:/ prefix to the filename on Android, if it is not started from resources.
    The suggested way to load a qml file is from resources, because on Android the file selectors with assets dir do not work!
   */
  void setMainQmlFileName(const QString &qmlFileName);

signals:

public slots:
  void objectCreatedHandler(QObject *object, const QUrl &url);

private:
  QQmlEngine *m_engine;
  VPlayScheduler *m_vplayScheduler;
  VPStepDriver *m_stepDriver;
  System *m_system;
  QStringList m_fileSelectorList;
  QString m_mainQmlFileName;
  GlobalKeyForwarder *m_globalKeyForwarder;
  VPlayFileUtils *m_fileUtils;

  VPlayUrlInterceptor *m_vplayUrlInterceptor;
  QFileSelector *m_vplayFileSelector;
  qreal m_contentScale;
  qreal m_scaleFactorForImages;
  QVariantMap m_contentScaleThresholds;

  /*!
   * Is called from initializeEngine() and sets the default contentScale and fileSelectors based on the screen size.
   */
  void setDefaultContentScalingAndFileSelectors();

  void addCustomImportPaths();
  QString adjustPath(const QString &path);
  QString adjustImportPathToCanonical(const QString &path);
  void addImportPath(const QString &path);
};

#endif // VPAPPLICATION_H
