import QtQuick 2.0
import VPlay 2.0

Item {
  property string licenseKey
  property string secret
  property int version
  property string androidPublicKey
  property bool supported

  property list<Currency> currencies
  property list<CurrencyPack> currencyPacks
  property list<Good> goods
  property list<NonConsumableGood> nonConsumables

  signal currencyBalanceChanged(string itemId, int balance, int amountAdded)
  signal goodBalanceChanged(string itemId, int balance, int amountAdded)
  signal goodEquipped(string itemId)
  signal goodUnequipped(string itemId)
  signal goodUpgrade(string itemId, string upgradeItemId)
  signal itemPurchased(string itemId)
  signal storePurchaseCanceled(string itemId)
  signal storePurchaseCancelled(string itemId)
  signal storePurchased(string itemId)
  signal storePurchaseStarted(string itemId)
  signal storeRefund(string itemId)
  signal restoreAllTransactionsStarted()
  signal restoreAllTransactionsFinished(bool success)
  signal unexpectedError()
  signal itemNotFoundError(string itemId)
  signal insufficientFundsError()


  function restoreAllTransactions() { }

  // Non consumables
  function nonConsumableItemExists(itemId) { }
  function addNonConsumableItem(itemId) { }
  function removeNonConsumableItem(itemId) { }

  // In-game items
  function buyItem(itemId) {
    __buyStoreItem(itemId)
  }

  function getItemBalance(itemId) {
    var item = __findItemById(itemId)

    if(!item) {
      itemNotFoundError()
      return
    }

    return item.balance
  }

  function giveItem(itemId, amount) {
    var item = __findCurrencyItemById(itemId)

    if(!item) {
      itemNotFoundError()
      return
    }

    item.balance += amount
    fakeStore.setValue(itemId, item.balance)
    currencyBalanceChanged(itemId, item.balance, amount)

  }

  function takeItem(itemId, amount) {
    var item = __findCurrencyItemById(itemId)

    if(!item) {
      itemNotFoundError()
      return
    }

    if(item.balance >= amount) {
      item.balance -= amount
      fakeStore.setValue(itemId, item.balance)
      currencyBalanceChanged(itemId, item.balance, -amount)
    } else {
      insufficientFundsError()
    }
  }

  // Used for easy creation of csv files to upload to dev consoles
  function printStoreProductLists() {
    console.debug("================================================================================\n")
    console.debug("Google Play CSV file:\n")
    for (var i = 0; i < goods.length; i++) {
      __printAndroidLineFromGood(goods[i])
    }
    for (i = 0; i < nonConsumables.length; i++) {
      __printAndroidLineFromGood(nonConsumables[i])
    }
    for (i = 0; i < currencyPacks.length; i++) {
      __printAndroidLineFromGood(currencyPacks[i])
    }
    console.debug("\niTunes Connect CSV file (be sure to replace your game's SKU and the item's price tier):\n")
    console.debug("SKU\tProduct ID\tReference Name\tType\tCleared For Sale\tWholesale Price Tier\tDisplayed Name\tDescription\tScreenshot Path\tEffective Date\tEnd Date")
    for (i = 0; i < goods.length; i++) {
      __printITunesLineFromGood(goods[i])
    }
    for (i = 0; i < nonConsumables.length; i++) {
      __printITunesLineFromGood(nonConsumables[i])
    }
    for (i = 0; i < currencyPacks.length; i++) {
      __printITunesLineFromGood(currencyPacks[i])
    }
    console.debug("\n================================================================================")
  }

  function __printAndroidLineFromGood(good) {
    var purchase = good.purchaseType
    if(purchase.objectName === "StorePurchase") {
      var row =
          purchase.productId +
          "," +
          "published" +
          "," +
          (good.objectName === "NonConsumableGood" ? "managed_by_android" : "managed_by_publisher") +
          "," +
          "false" +
          "," +
          "en_US" +
          ";" +
          good.name +
          ";" +
          good.description +
          "," +
          "true" +
          "," +
          parseInt(purchase.price * 1000000)

      console.debug(row)
    }
  }

  function __printITunesLineFromGood(good) {
    var purchase = good.purchaseType
    if(purchase.objectName === "StorePurchase") {
      var row =
          "[SKU]" +
          "\t" +
          purchase.productId +
          "\t" +
          purchase.productId +
          "\t" +
          (good.objectName === "NonConsumableGood" ? "Non-Consumable" : "Consumable") +
          "\t" +
          "Cleared For Sale" +
          "\t" +
          "[PRICE TIER]" +
          "\t" +
          good.name +
          "\t" +
          good.description +
          "\t" +
          "" +
          "\t" +
          "now" +
          "\t" +
          "none"

      console.debug(row)
    }
  }


  // Store calls for testing on platforms not supported by store, but which still simulate the most important functionality
  function __buyStoreItem(itemId) {
    storePurchaseStarted(itemId)

    // TODO: add a debug variable to simulate a failed purchase here

    var currencyPack = __findCurrencyPackItemById(itemId)
    if(currencyPack) {
      var currencyId = currencyPack.currencyId
      // add this to the local balance
      giveItem(currencyId, currencyPack.currencyAmount)
    }


    storePurchased(itemId)
    itemPurchased(itemId)

  }

  function __findCurrencyItemById(itemId) {

    for(var i=0; i<currencies.length; i++) {
      var currencyItem = currencies[i]

      if(itemId === currencyItem.itemId) {
        return currencyItem
      }
    }
    return undefined

  }

  function __findCurrencyPackItemById(itemId) {

    for(var i=0; i<currencyPacks.length; i++) {
      var currencyPackItem = currencyPacks[i]

      if(itemId === currencyPackItem.itemId) {
        return currencyPackItem
      }
    }
    return undefined

  }

  // by storing the balance values of the currencies locally, we can simulate the Store behavior
  Storage {
    id: fakeStore
    databaseName: "___localStore___"
    Component.onCompleted: {

      // load the balance values from the database at startup
      for(var i=0; i<currencies.length; i++) {
        var currencyItem = currencies[i]
        var balance = getValue(currencyItem.itemId)

        // a if(!balance) would be wrong, because balance can also be 0
        if(balance !== undefined) {
          currencyItem.balance = balance
        }
      }
    }
  }

}
